from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.db import IntegrityError
from django.db.models.query import QuerySet
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.views import generic

from .models import Comment, Post, Tag, Vote

class HomeView(generic.ListView):
    template_name = "index.html"
    context_object_name = "posts"

    def get_queryset(self):
        return Post.objects.order_by("-created_at")
    
# Add your own views here

#############################
# USER LOGIN IMPLEMENTATION #
#############################

# You shouldn't need to touch anything below here, but it may be helpful to 
# see what's going on.

def userlogin(request):
    if request.method == "GET":
        return render(request, "login.html")
    username = request.POST["username"]
    password = request.POST["password"]
    user = authenticate(request, username=username, password=password)
    if user is not None:
        login(request, user)
    else:
        return render(
            request,
            "login.html",
            {
                "message": "Invalid login credentials.",
            },
        )
    return HttpResponseRedirect(reverse("blog:index"))

def userlogout(request):
    logout(request)
    return HttpResponseRedirect(reverse("blog:index"))

def usersignup(request):
    if request.method == "GET":
        return render(request, "signup.html")
    username = request.POST["username"]
    password = request.POST["password"]
    fname = request.POST["fname"]
    lname = request.POST["lname"]

    if not username or not password or not fname or not lname:
        return render(
            request,
            "signup.html",
            {
                "message": "Missing fields for user signup.",
            },
        )

    try:
        u = User.objects.create_user(username, username, password, first_name=fname, last_name=fname)
        login(request, u)
        return HttpResponseRedirect(reverse("blog:index"))
    except IntegrityError:
        message = "Username is taken already."
        return render(request, "signup.html", {"message": message})
    except Exception as e:
        message = f"Failed user signup: {e}"
        return render(request, "signup.html", {"message": message})
